package com.jjt.hl.controller;

import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.core.page.TableDataInfo;
import com.jjt.common.enums.BusinessType;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.hl.domain.HlMetricsScore;
import com.jjt.hl.service.IHlMetricsScoreService;
import com.jjt.hl.vo.ScoreDetailVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;

/**
 * 业务模型健康度指标得分Controller
 *
 * @author jjt
 * @date 2024-09-05
 */
@Api(tags = "业务模型健康度指标得分")
@RestController
@RequestMapping("/hl/hms")
public class HlMetricsScoreController extends BaseController {
    @Resource
    private IHlMetricsScoreService hlMetricsScoreService;

    /**
     * 查询模型健康度分类得分列表
     */
    @ApiOperation("根据分类ID查询详细")
    @GetMapping("/list/{scoreClassId}")
    public AjaxResult list4score(@PathVariable("scoreClassId") Long scoreClassId) {
        HlMetricsScore hlMetricsScore = new HlMetricsScore();
        hlMetricsScore.setScoreClassId(scoreClassId);
        List<HlMetricsScore> list = hlMetricsScoreService.selectHlMetricsScoreList(hlMetricsScore);
        return AjaxResult.success(list);
    }

    /**
     * 查询模型健康度分类得分列表
     */
    @ApiOperation("根据指标查询扣分明细")
    @GetMapping("/detail/{scoreMetricsId}")
    public AjaxResult detail4score(@PathVariable("scoreMetricsId") Long scoreMetricsId) {
        //TODO 到处都是模拟数据
//        String[] names = {"cluster-node1", "cluster-node2", "cluster-node3", "cluster-node4", "cluster-node5"};
//        String[] remarks = {"超过80%，且持续1小时", "超过60，且持续1小时", "超过90%", "超过200，且持续1小时", "当前值超过5"};
        Random random = new Random();
        List<ScoreDetailVO> list = new ArrayList<>();
        for (int i = 0; i < 5; i++) {
            ScoreDetailVO vo = new ScoreDetailVO();
            vo.setObjName("node-" + random.nextInt(50));
            vo.setEvent("超过80,且持续1小时");
            vo.setMetricsName("内存使用率");
            float v = 80F + random.nextInt(20);
            vo.setValue(v);
            vo.setTime(new Date());
            list.add(vo);
        }
        return AjaxResult.success(list);
    }

    /**
     * 查询业务模型健康度指标得分列表
     */
    @ApiOperation("查询业务模型健康度指标得分列表")
    @PreAuthorize("@ss.hasPermi('hl:hms:list')")
    @GetMapping("/list")
    public TableDataInfo list(HlMetricsScore hlMetricsScore) {
        startPage();
        List<HlMetricsScore> list = hlMetricsScoreService.selectHlMetricsScoreList(hlMetricsScore);
        return getDataTable(list);
    }

    /**
     * 导出业务模型健康度指标得分列表
     */
    @ApiOperation("导出业务模型健康度指标得分列表")
    @PreAuthorize("@ss.hasPermi('hl:hms:export')")
    @Log(title = "业务模型健康度指标得分", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, HlMetricsScore hlMetricsScore) {
        List<HlMetricsScore> list = hlMetricsScoreService.selectHlMetricsScoreList(hlMetricsScore);
        ExcelUtil<HlMetricsScore> util = new ExcelUtil<HlMetricsScore>(HlMetricsScore.class);
        util.exportExcel(response, list, "业务模型健康度指标得分数据");
    }

    /**
     * 获取业务模型健康度指标得分详细信息
     */
    @ApiOperation("获取业务模型健康度指标得分详细信息")
    @PreAuthorize("@ss.hasPermi('hl:hms:query')")
    @GetMapping(value = "/{scoreMetricsId}")
    public AjaxResult getInfo(@PathVariable("scoreMetricsId") Long scoreMetricsId) {
        return success(hlMetricsScoreService.selectHlMetricsScoreByScoreMetricsId(scoreMetricsId));
    }

    /**
     * 新增业务模型健康度指标得分
     */
    @ApiOperation("新增业务模型健康度指标得分")
    @PreAuthorize("@ss.hasPermi('hl:hms:add')")
    @Log(title = "业务模型健康度指标得分", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody HlMetricsScore hlMetricsScore) {
        return toAjax(hlMetricsScoreService.insertHlMetricsScore(hlMetricsScore));
    }

    /**
     * 修改业务模型健康度指标得分
     */
    @ApiOperation("修改业务模型健康度指标得分")
    @PreAuthorize("@ss.hasPermi('hl:hms:edit')")
    @Log(title = "业务模型健康度指标得分", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody HlMetricsScore hlMetricsScore) {
        return toAjax(hlMetricsScoreService.updateHlMetricsScore(hlMetricsScore));
    }

    /**
     * 删除业务模型健康度指标得分
     */
    @ApiOperation("删除业务模型健康度指标得分")
    @PreAuthorize("@ss.hasPermi('hl:hms:remove')")
    @Log(title = "业务模型健康度指标得分", businessType = BusinessType.DELETE)
    @DeleteMapping("/{scoreMetricsIds}")
    public AjaxResult remove(@PathVariable Long[] scoreMetricsIds) {
        return toAjax(hlMetricsScoreService.deleteHlMetricsScoreByScoreMetricsIds(scoreMetricsIds));
    }
}
