package com.jjt.risk.controller;

import com.jjt.biz.domain.BizModel;
import com.jjt.biz.service.IBizModelService;
import com.jjt.biz.vo.ScoreVO;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.risk.domain.RiskMsConfig;
import com.jjt.risk.service.IRiskAnalysisService;
import com.jjt.risk.service.IRiskModelService;
import com.jjt.risk.service.IRiskMsConfigService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;

/**
 * 风险分析结果Controller
 *
 * @author jjt
 * @date 2024-09-12
 */
@Api(tags = "其他风险分析")
@RestController
@RequestMapping("/risk/other")
public class RiskOtherController extends BaseController {
    @Resource
    private IRiskAnalysisService riskAnalysisService;
    @Resource
    private IRiskModelService riskModelService;
    @Resource
    private IBizModelService bizModelService;
    @Resource
    private IRiskMsConfigService riskMsConfigService;

    @ApiOperation("模型列表")
    @GetMapping(value = "/model/list")
    public AjaxResult modelList() {
        List<BizModel> list = bizModelService.selectBizModelList(new BizModel());
        return success(list);
    }

    @ApiOperation("业务主机分析")
    @GetMapping(value = "/host/{modelId}")
    public AjaxResult host(@PathVariable("modelId") Long modelId) {
        BizModel model = bizModelService.selectBizModelByModelId(modelId);
        RiskMsConfig q = new RiskMsConfig();
        q.setConfigType("host");
        List<RiskMsConfig> configs = riskMsConfigService.selectRiskMsConfigList(q);
        String[] objs = {"ecs1", "ecs2", "ecs3", "ecs4", "ecs5", "ecs6", "ecs7"};
        List<Map<String, Object>> result = new ArrayList<>();
        for (RiskMsConfig config : configs) {
            Map<String, Object> map = new HashMap<>(16);
            map.put("title", config.getViewName());
            List<Map<String, Object>> dataList = new ArrayList<>();
            for (String obj : objs) {
                Map<String, Object> dataMap = new HashMap<>(16);
                String name = model.getModelName() + "-" + obj;
                dataMap.put("objName", name);
                dataMap.put("value", new Random().nextInt(100));
                if ("2".equals(config.getRankingBased())) {
                    dataMap.put("alarms", new Random().nextInt(50));
                }
                dataList.add(dataMap);
            }
            map.put("data", dataList);
            result.add(map);
        }

        return success(result);
    }

    @ApiOperation("网络状况分析")
    @GetMapping(value = "/network/{modelId}/{metricsId}")
    public AjaxResult network(@PathVariable("modelId") Long modelId, @PathVariable("metricsId") Long metricsId) {
        BizModel model = bizModelService.selectBizModelByModelId(modelId);
        RiskMsConfig q = new RiskMsConfig();
        q.setConfigType("host");
        List<Map<String, Object>> result = new ArrayList<>();
        String[] objs = {"node1", "node2", "node3", "ecs4", "业务对象5", "虚拟主机6", "cluster7"};
        for (String obj : objs) {
            Map<String, Object> map = new HashMap<>(16);
            map.put("name", obj);

            List<Map<String, Object>> dataList = new ArrayList<>();

            LocalDateTime ed = LocalDateTime.now();
            LocalDateTime st = ed.minusDays(30);
            Random r = new Random();
            DecimalFormat df = new DecimalFormat("#0.00");
            do {
                Map<String, Object> dataMap = new HashMap<>(16);
                long time = st.toEpochSecond(ZoneOffset.ofHours(8)) * 1000;
                dataMap.put("time", time);
                float f = r.nextFloat() * 100;
                dataMap.put("value", Float.parseFloat(df.format(f)));
                st = st.plusDays(1);
                dataList.add(dataMap);
            } while (!st.isAfter(ed));
            map.put("data", dataList);

            result.add(map);
        }
        return success(result);
    }

    @ApiOperation("可持续性服务评估")
    @GetMapping(value = "/server")
    public AjaxResult server() {
        Map<String, Object> result = new HashMap<>(16);
        List<ScoreVO> list = new ArrayList<>();
        List<Map<String, Object>> topList = new ArrayList<>();
        String[] names = {"市场服务", "市场出清", "信息发布", "市场合规", "市场结算"};
        for (int i = 0; i < names.length; i++) {
            Map<String, Object> map = new HashMap<>(16);
            map.put("name", names[i]);
            map.put("score", Float.valueOf(new Random().nextInt(50) + 50));
            topList.add(map);
            ScoreVO vo = new ScoreVO();
            List<String> xData = new ArrayList<>();
            List<Float> scores = new ArrayList<>();
            LocalDate time = LocalDate.now().minusDays(31);
            for (int j = 0; j < 30; j++) {
                time = time.plusDays(1);
                xData.add(time.getMonthValue() + "-" + time.getDayOfMonth());
                Float score = Float.valueOf(new Random().nextInt(50) + 50);
                scores.add(score);
            }

            vo.setModelId((long) (i + 1));
            vo.setModelName(names[i]);
            vo.setScores(scores);
            vo.setXData(xData);
            list.add(vo);
        }
        topList.sort(Comparator.comparing(map -> (Float) map.get("score")));

        result.put("trend", list);
        result.put("top", topList);
        return success(result);
    }
}
