package com.jjt.risk.service.impl;

import java.util.List;
        import com.jjt.common.utils.DateUtils;
import org.springframework.stereotype.Service;
import com.jjt.risk.mapper.RiskAnalysisMapper;
import com.jjt.risk.domain.RiskAnalysis;
import com.jjt.risk.service.IRiskAnalysisService;
import javax.annotation.Resource;

/**
 * 风险分析结果Service业务层处理
 *
 * @author jjt
 * @date 2024-09-12
 */
@Service
public class RiskAnalysisServiceImpl implements IRiskAnalysisService {
    @Resource
    private RiskAnalysisMapper riskAnalysisMapper;

    /**
     * 查询风险分析结果
     *
     * @param resultId 风险分析结果主键
     * @return 风险分析结果
     */
    @Override
    public RiskAnalysis selectRiskAnalysisByResultId(Long resultId) {
        return riskAnalysisMapper.selectRiskAnalysisByResultId(resultId);
    }

    /**
     * 查询风险分析结果列表
     *
     * @param riskAnalysis 风险分析结果
     * @return 风险分析结果
     */
    @Override
    public List<RiskAnalysis> selectRiskAnalysisList(RiskAnalysis riskAnalysis) {
        return riskAnalysisMapper.selectRiskAnalysisList(riskAnalysis);
    }

    /**
     * 新增风险分析结果
     *
     * @param riskAnalysis 风险分析结果
     * @return 结果
     */
    @Override
    public int insertRiskAnalysis(RiskAnalysis riskAnalysis) {
                riskAnalysis.setCreateTime(DateUtils.getNowDate());
            return riskAnalysisMapper.insertRiskAnalysis(riskAnalysis);
    }

    /**
     * 修改风险分析结果
     *
     * @param riskAnalysis 风险分析结果
     * @return 结果
     */
    @Override
    public int updateRiskAnalysis(RiskAnalysis riskAnalysis) {
                riskAnalysis.setUpdateTime(DateUtils.getNowDate());
        return riskAnalysisMapper.updateRiskAnalysis(riskAnalysis);
    }

    /**
     * 批量删除风险分析结果
     *
     * @param resultIds 需要删除的风险分析结果主键
     * @return 结果
     */
    @Override
    public int deleteRiskAnalysisByResultIds(Long[] resultIds) {
        return riskAnalysisMapper.deleteRiskAnalysisByResultIds(resultIds);
    }

    /**
     * 删除风险分析结果信息
     *
     * @param resultId 风险分析结果主键
     * @return 结果
     */
    @Override
    public int deleteRiskAnalysisByResultId(Long resultId) {
        return riskAnalysisMapper.deleteRiskAnalysisByResultId(resultId);
    }
}
